=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2006-04-19 23:32:08 +0000
+++ bzrlib/repository.py	2006-05-04 11:17:36 +0000
@@ -116,6 +116,14 @@
         result = self._all_possible_ids()
         return self._eliminate_revisions_not_present(result)
 
+    def break_lock(self):
+        """Break a lock if one is present from another instance.
+
+        Uses the ui factory to ask for confirmation if the lock may be from
+        an active process.
+        """
+        self.control_files.break_lock()
+
     @needs_read_lock
     def _eliminate_revisions_not_present(self, revision_ids):
         """Check every revision id in revision_ids to see if we have it.

=== modified file 'bzrlib/tests/repository_implementations/__init__.py'
--- bzrlib/tests/repository_implementations/__init__.py	2006-02-23 04:08:56 +0000
+++ bzrlib/tests/repository_implementations/__init__.py	2006-05-04 11:17:36 +0000
@@ -40,6 +40,7 @@
 def test_suite():
     result = TestSuite()
     test_repository_implementations = [
+        'bzrlib.tests.repository_implementations.test_break_lock',
         'bzrlib.tests.repository_implementations.test_fileid_involved',
         'bzrlib.tests.repository_implementations.test_reconcile',
         'bzrlib.tests.repository_implementations.test_repository',

=== added file 'bzrlib/tests/repository_implementations/test_break_lock.py'
--- bzrlib/tests/repository_implementations/test_break_lock.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/repository_implementations/test_break_lock.py	2006-05-04 11:17:36 +0000
@@ -0,0 +1,61 @@
+# Copyright (C) 2006 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for repository break-lock."""
+
+from cStringIO import StringIO
+
+import bzrlib
+import bzrlib.errors as errors
+from bzrlib.tests.repository_implementations.test_repository import TestCaseWithRepository
+from bzrlib.transport import get_transport
+from bzrlib.tree import EmptyTree
+from bzrlib.workingtree import WorkingTree
+
+
+class TestBreakLock(TestCaseWithRepository):
+
+    def setUp(self):
+        super(TestBreakLock, self).setUp()
+        self.unused_repo = self.make_repository('.')
+        self.repo = self.unused_repo.bzrdir.open_repository()
+        # we want a UI factory that accepts canned input for the tests:
+        # while SilentUIFactory still accepts stdin, we need to customise
+        # ours
+        self.old_factory = bzrlib.ui.ui_factory
+        bzrlib.ui.ui_factory = bzrlib.ui.SilentUIFactory()
+        bzrlib.ui.ui_factory.stdin = StringIO("y\n")
+
+    def tearDown(self):
+        bzrlib.ui.ui_factory = self.old_factory
+
+    def test_unlocked(self):
+        # break lock when nothing is locked should just return
+        try:
+            self.repo.break_lock()
+        except NotImplementedError:
+            pass
+
+    def test_locked(self):
+        # break_lock when locked should
+        self.repo.lock_write()
+        try:
+            self.unused_repo.break_lock()
+        except NotImplementedError:
+            # repository does not support break_lock
+            self.repo.unlock()
+            return
+        self.assertRaises(errors.LockBroken, self.repo.unlock)

